@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,6 +27,7 @@
 #define ROCWMMA_CONVERT_HPP
 
 #include "types.hpp"
+#include "utility/forward.hpp"
 
 namespace rocwmma
 {
@@ -37,7 +38,7 @@ namespace rocwmma
         template <typename InputT, typename OutputT>
         struct amdgcn_convert
         {
-            template <uint32_t NumRegs>
+            template <template <typename, uint32_t> class VecT, uint32_t NumRegs>
             ROCWMMA_DEVICE static inline auto exec(VecT<InputT, NumRegs> const& regsIn)
                 -> VecT<OutputT, NumRegs>
             {
@@ -46,7 +47,7 @@ namespace rocwmma
 #pragma unroll
                 for(unsigned i = 0; i < NumRegs; i++)
                 {
-                    result.data[i] = static_cast<OutputT>(regsIn.data[i]);
+                    get<i>(result) = static_cast<OutputT>(get<i>(regsIn));
                 }
                 return result;
             }
@@ -56,9 +57,9 @@ namespace rocwmma
         struct amdgcn_convert<T, T>
         {
             template <typename IncomingT>
-            ROCWMMA_DEVICE static inline auto exec(IncomingT&& regsIn) -> IncomingT&&
+            ROCWMMA_DEVICE static inline decltype(auto) exec(IncomingT&& regsIn)
             {
-                return std::forward<IncomingT>(regsIn);
+                return forward<IncomingT>(regsIn);
             }
         };
 
@@ -66,16 +67,15 @@ namespace rocwmma
         template <>
         struct amdgcn_convert<hfloat16_t, float32_t>
         {
-            template <uint32_t NumRegs>
+            template <template <typename, uint32_t> class VecT, uint32_t NumRegs>
             ROCWMMA_DEVICE static inline auto exec(VecT<hfloat16_t, NumRegs> const& regsIn)
-                -> VecT<float32_t, NumRegs>
             {
                 VecT<float32_t, NumRegs> result;
 
 #pragma unroll
                 for(unsigned i = 0; i < NumRegs; i++)
                 {
-                    result.data[i] = __half2float(regsIn.data[i]);
+                    get<i>(result) = __half2float(get<i>(regsIn));
                 }
                 return result;
             }
@@ -84,16 +84,15 @@ namespace rocwmma
         template <>
         struct amdgcn_convert<float32_t, hfloat16_t>
         {
-            template <uint32_t NumRegs>
+            template <template <typename, uint32_t> class VecT, uint32_t NumRegs>
             ROCWMMA_DEVICE static inline auto exec(VecT<float32_t, NumRegs> const& regsIn)
-                -> VecT<hfloat16_t, NumRegs>
             {
                 VecT<hfloat16_t, NumRegs> result;
 
 #pragma unroll
                 for(unsigned i = 0; i < NumRegs; i++)
                 {
-                    result.data[i] = __float2half(regsIn.data[i]);
+                    get<i>(result) = __float2half(get<i>(regsIn));
                 }
                 return result;
             }
