@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -137,67 +137,6 @@ namespace std
 
 namespace rocwmma
 {
-    // Computes ceil(numerator/divisor) for integer types.
-    template <typename intT1,
-              class = typename enable_if<is_integral<intT1>::value>::type,
-              typename intT2,
-              class = typename enable_if<is_integral<intT2>::value>::type>
-    static constexpr intT1 ceilDiv(const intT1 numerator, const intT2 divisor)
-    {
-        return (numerator + divisor - 1) / divisor;
-    }
-
-    // Calculate integer Log base 2
-    template <uint32_t x>
-    struct Log2
-    {
-        static constexpr uint32_t value = 1 + Log2<(x >> 1)>::value;
-        static_assert(x % 2 == 0, "Integer input must be a power of 2");
-    };
-
-    template <>
-    struct Log2<1>
-    {
-        static constexpr uint32_t value = 0;
-    };
-
-    template <>
-    struct Log2<0>
-    {
-        static constexpr uint32_t value = 0;
-    };
-
-    // Create a bitmask of size BitCount, starting from the LSB bit
-    template <uint32_t BitCount>
-    struct LsbMask;
-
-    template <>
-    struct LsbMask<1>
-    {
-        enum : uint32_t
-        {
-            value = 0x1
-        };
-    };
-
-    template <>
-    struct LsbMask<0>
-    {
-        enum : uint32_t
-        {
-            value = 0x0
-        };
-    };
-
-    template <uint32_t BitCount>
-    struct LsbMask
-    {
-        enum : uint32_t
-        {
-            value = LsbMask<1>::value << (BitCount - 1) | LsbMask<BitCount - 1>::value
-        };
-    };
-
     // Helper for string representations of types
     template <typename DataT>
     constexpr const char* dataTypeToString();
