@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -196,8 +196,8 @@ __host__ void sgemv_test(uint32_t m, uint32_t n, uint32_t k, float alpha, float
     CHECK_HIP_ERROR(hipMemcpy(d_c, matrixC.data(), bytesC, hipMemcpyHostToDevice));
 
     auto blockDim = dim3(T_BLOCK_X, T_BLOCK_Y);
-    auto gridDim  = dim3(rocwmma::ceilDiv(m, ROCWMMA_M * T_BLOCK_X / WAVE_SIZE),
-                        rocwmma::ceilDiv(n, ROCWMMA_N * T_BLOCK_Y));
+    auto gridDim  = dim3(rocwmma::ceil_div(m, ROCWMMA_M * T_BLOCK_X / WAVE_SIZE),
+                        rocwmma::ceil_div(n, ROCWMMA_N * T_BLOCK_Y));
 
     std::cout << "Launching sgemv kernel..." << std::endl;
     hipEvent_t startEvent, stopEvent;
