@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -72,19 +72,51 @@
 
 namespace rocwmma
 {
+    ROCWMMA_DEVICE inline bool isFirstThread()
+    {
+        return (threadIdx.x == 0) && (threadIdx.y == 0) && (threadIdx.z == 0) && (blockIdx.x == 0)
+               && (blockIdx.y == 0) && (blockIdx.z == 0);
+    }
+
+    template <typename Lhs, typename Rhs>
+    ROCWMMA_DEVICE bool
+        expectEqual(Lhs&& testL, Rhs&& testR, bool debugOnFail, const char* file, uint32_t line)
+    {
+        bool compare_result = (testL == testR);
+        if(!compare_result && debugOnFail && isFirstThread())
+        {
+            printf("<TestBegin>\n");
+            printf("File: %s L:%d\n", file, line);
+            if constexpr(is_same_v<decay_t<Lhs>, hfloat16_t>)
+            {
+                printf("Lhs: %f, Rhs: %f\n", static_cast<float>(testL), static_cast<float>(testR));
+            }
+            printf("FAILED");
+            printf("<Test End>\n");
+        }
+
+        return compare_result;
+    }
+
+    template <typename Lhs, typename Rhs>
+    ROCWMMA_DEVICE bool
+        expectNotEqual(Lhs&& testL, Rhs&& testR, bool debugOnFail, const char* file, uint32_t line)
+    {
+        bool compare_result = (testL != testR);
+        if(!compare_result && debugOnFail && isFirstThread())
+        {
+            printf("<TestBegin>\n");
+            printf("File: %s L:%d\n", file, line);
+            printf("FAILED");
+            printf("<Test End>\n");
+        }
+
+        return compare_result;
+    }
+
     static constexpr uint32_t ERROR_VALUE   = 7u;
     static constexpr uint32_t SUCCESS_VALUE = 0u;
 
-    template <uint32_t N>
-    using I = std::integral_constant<uint32_t, N>;
-
-    template <class... Ts, class F>
-    void for_each(std::tuple<Ts...>, F f)
-    {
-        std::initializer_list<int> _ = {(f(Ts{}), 0)...}; // poor man's fold expression for C++11/14
-        // (f(Ts{}), ...); // fold expression is for C++17 only
-    }
-
     namespace quirks
     {
         // rocBLAS does not yet support Ti/To/Tc = bf16/bf16/bf16
@@ -213,8 +245,8 @@ namespace rocwmma
                         // Count up in integers, in ascending order for each row.
                         auto value = ((i - padM) * n + (j - padN)) % 5;
                         mat[idx]   = ((value % 3) && std::is_signed<DataT>::value)
-                                         ? -static_cast<DataT>(value)
-                                         : static_cast<DataT>(value);
+                                       ? -static_cast<DataT>(value)
+                                       : static_cast<DataT>(value);
                     }
                 }
             }
@@ -240,7 +272,7 @@ namespace rocwmma
             const auto limitN = n + 2 * padN;
 
             auto blockDim = dim3(1024, 1, 1);
-            auto gridDim  = dim3(ceilDiv(limitM * limitN, blockDim.x), 1, 1);
+            auto gridDim  = dim3(ceil_div(limitM * limitN, blockDim.x), 1, 1);
             hipLaunchKernelGGL((fillWithPaddingKernel<DataT, Layout>),
                                gridDim,
                                blockDim,
@@ -273,8 +305,8 @@ namespace rocwmma
                     auto value = (i * n + j) % 5;
                     auto idx   = index(i, j, ld);
                     mat[idx]   = ((value % 3) && std::is_signed<DataT>::value)
-                                     ? -static_cast<DataT>(value)
-                                     : static_cast<DataT>(value);
+                                   ? -static_cast<DataT>(value)
+                                   : static_cast<DataT>(value);
                 }
             }
         }
@@ -309,7 +341,7 @@ namespace rocwmma
         __host__ static inline void fillLaunchKernel(DataT* d_mat, uint32_t m, uint32_t n)
         {
             auto blockDim = dim3(1024, 1, 1);
-            auto gridDim  = dim3(ceilDiv(m * n, blockDim.x), 1, 1);
+            auto gridDim  = dim3(ceil_div(m * n, blockDim.x), 1, 1);
             hipLaunchKernelGGL((fillKernel<DataT, Layout>), gridDim, blockDim, 0, 0, d_mat, m, n);
         }
 
@@ -319,7 +351,7 @@ namespace rocwmma
             fillLaunchKernel(DataT* d_mat, uint32_t m, uint32_t k, uint32_t b)
         {
             auto blockDim = dim3(1024, 1, 1);
-            auto gridDim  = dim3(ceilDiv(m * k, blockDim.x), 1, b);
+            auto gridDim  = dim3(ceil_div(m * k, blockDim.x), 1, b);
             hipLaunchKernelGGL(
                 (fillKernel<DataT, Layout>), gridDim, blockDim, 0, 0, d_mat, m, k, b);
         }
@@ -330,7 +362,7 @@ namespace rocwmma
             fillValLaunchKernel(DataT* d_mat, uint32_t m, uint32_t n, DataT value)
         {
             auto blockDim = dim3(1024, 1, 1);
-            auto gridDim  = dim3(ceilDiv(m * n, blockDim.x), 1, 1);
+            auto gridDim  = dim3(ceil_div(m * n, blockDim.x), 1, 1);
             hipLaunchKernelGGL(
                 (fillValKernel<DataT, Layout>), gridDim, blockDim, 0, 0, d_mat, m, n, value);
         }
@@ -340,7 +372,7 @@ namespace rocwmma
         __host__ static inline void fillIdxLaunchKernel(DataT* d_mat, uint32_t m, uint32_t n)
         {
             auto blockDim = dim3(1024, 1, 1);
-            auto gridDim  = dim3(ceilDiv(m * n, blockDim.x), 1, 1);
+            auto gridDim  = dim3(ceil_div(m * n, blockDim.x), 1, 1);
             hipLaunchKernelGGL(
                 (fillIdxKernel<DataT, Layout>), gridDim, blockDim, 0, 0, d_mat, m, n);
         }
@@ -579,7 +611,7 @@ namespace rocwmma
         uint32_t ldb = std::is_same<LayoutB, row_major>::value ? n : m;
 
         auto blockDim = dim3(1024, 1, 1);
-        auto gridDim  = dim3(ceilDiv(m * n, blockDim.x), 1, 1);
+        auto gridDim  = dim3(ceil_div(m * n, blockDim.x), 1, 1);
 
         double* d_relativeError;
         double  maxRelativeError;
@@ -609,9 +641,9 @@ namespace rocwmma
         uint32_t maxElements = 1024;
         uint32_t offset      = 1;
 
-        for(uint32_t i = m * n; i > 1; i = ceilDiv(i, maxElements))
+        for(uint32_t i = m * n; i > 1; i = ceil_div(i, maxElements))
         {
-            gridDim       = dim3(ceilDiv(i, maxElements), 1, 1);
+            gridDim       = dim3(ceil_div(i, maxElements), 1, 1);
             auto elements = i > maxElements ? maxElements : i;
 
             hipLaunchKernelGGL((maxReduceKernel),
@@ -661,7 +693,7 @@ namespace rocwmma
         TypeA* matrixA, TypeB* matrixB, uint32_t m, uint32_t k, uint32_t b, double tolerance = 10.0)
     {
         auto blockDim = dim3(1024, 1, 1);
-        auto gridDim  = dim3(ceilDiv(m * k, blockDim.x), 1, b);
+        auto gridDim  = dim3(ceil_div(m * k, blockDim.x), 1, b);
 
         double* d_relativeError;
         double  maxRelativeError;
@@ -690,9 +722,9 @@ namespace rocwmma
         uint32_t maxElements = 1024;
         uint32_t offset      = 1;
 
-        for(uint32_t i = m * k * b; i > 1; i = ceilDiv(i, maxElements))
+        for(uint32_t i = m * k * b; i > 1; i = ceil_div(i, maxElements))
         {
-            gridDim       = dim3(ceilDiv(i, maxElements), 1, 1);
+            gridDim       = dim3(ceil_div(i, maxElements), 1, 1);
             auto elements = i > maxElements ? maxElements : i;
 
             hipLaunchKernelGGL((maxReduceKernel),
