@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -81,14 +81,14 @@ namespace rocwmma
 
         if(passDirection == DlrmDirection_t::Forward)
         {
-            return dim3(ceilDiv(mMPadded, TileSize * mTBlockX / device->warpSize()),
-                        ceilDiv(mM, TileSize),
+            return dim3(ceil_div(mMPadded, TileSize * mTBlockX / device->warpSize()),
+                        ceil_div(mM, TileSize),
                         mB);
         }
         else
         {
-            return dim3(ceilDiv(mMPadded, TileSize * mTBlockX / device->warpSize()),
-                        ceilDiv(mK, TileSize),
+            return dim3(ceil_div(mMPadded, TileSize * mTBlockX / device->warpSize()),
+                        ceil_div(mK, TileSize),
                         mB);
         }
     }
@@ -108,7 +108,8 @@ namespace rocwmma
         // Arch
         auto isGfx908 = deviceArch == DeviceInfo::GFX908;
         auto isGfx11  = (deviceArch == DeviceInfo::GFX1100) || (deviceArch == DeviceInfo::GFX1101)
-                       || (deviceArch == DeviceInfo::GFX1102);
+                       || (deviceArch == DeviceInfo::GFX1102)
+                       || (deviceArch == DeviceInfo::GFX1151);
 
         auto isGfx12 = (deviceArch == DeviceInfo::GFX1200) || (deviceArch == DeviceInfo::GFX1201);
 
@@ -251,8 +252,8 @@ namespace rocwmma
                        static_cast<uint32_t const&>(std::get<1>(problem.problemSize)),
                        static_cast<uint32_t const&>(std::get<2>(problem.problemSize)));
 
-        mMPadded = ceilDiv(mM, TileSize) * TileSize;
-        mKPadded = ceilDiv(mK, TileSize) * TileSize;
+        mMPadded = ceil_div(mM, TileSize) * TileSize;
+        mKPadded = ceil_div(mK, TileSize) * TileSize;
 
         // Determine whether to run forward or backward pass
         passDirection = problem.passDirection;
@@ -347,7 +348,7 @@ namespace rocwmma
                     dlrmKernel = [this, inputBatchOffset, upstreamBatchOffset, accBatchOffset]() {
                         auto& dataInstance = DataStorage::instance();
                         auto  trilGridDim
-                            = dim3(ceilDiv(mM * mM, static_cast<uint32_t>(mTBlockX)), 1, mB);
+                            = dim3(ceil_div(mM * mM, static_cast<uint32_t>(mTBlockX)), 1, mB);
 
                         hipEvent_t syncEvent;
                         CHECK_HIP_ERROR(hipEventCreate(&syncEvent));
