@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -122,10 +122,10 @@ namespace rocwmma
 
         dim3 gridDim() const final
         {
-            return dim3(ceilDiv(Base::mM,
-                                BlockM * BlocksX * Base::mTBlockX
-                                    / Base::DeviceInfo::instance()->warpSize()),
-                        ceilDiv(Base::mN, BlockN * BlocksY * Base::mTBlockY));
+            return dim3(ceil_div(Base::mM,
+                                 BlockM * BlocksX * Base::mTBlockX
+                                     / Base::DeviceInfo::instance()->warpSize()),
+                        ceil_div(Base::mN, BlockN * BlocksY * Base::mTBlockY));
         }
 
         bool checkSizes() const final
