@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -35,7 +35,6 @@
 #include "gemm_config.hpp"
 #include "kernel_predicates.hpp"
 #include <rocwmma/rocwmma.hpp>
-#include <rocwmma/rocwmma_coop.hpp>
 #pragma GCC diagnostic pop
 
 namespace rocwmma
@@ -119,10 +118,7 @@ namespace rocwmma
                                                                               TBlockY>;
 
             using LdsMapping = typename GemmConfig::template LdsMapping<GlobalMapping, LayoutLds>;
-            using CoopSchedulerA = typename GemmConfig::template CoopSchedulerA<TBlockX, TBlockY>;
-            using CoopSchedulerB = typename GemmConfig::template CoopSchedulerB<TBlockX, TBlockY>;
-            using GemmDriver     = typename GemmConfig::
-                template GemmDriver<GlobalMapping, LdsMapping, CoopSchedulerA, CoopSchedulerB>;
+            using GemmDriver = typename GemmConfig::template GemmDriver<GlobalMapping, LdsMapping>;
 
             // Global fragments used in pre-fetching
             using GRFragA = typename GlobalMapping::GRFragA;
@@ -246,13 +242,15 @@ namespace rocwmma
                 GemmDriver::localWriteCoopA(ldsPtrHi + ldsWriteOffsetA, grBuffA, ldlds);
                 GemmDriver::localWriteCoopB(ldsPtrHi + ldsWriteOffsetB, grBuffB, ldlds);
 
-                // Make sure that all waves have finished reading / writing to lds.
-                GemmDriver::syncWorkgroup();
-
                 // Swap Lds buffers
                 auto* tmp = ldsPtrLo;
                 ldsPtrLo  = ldsPtrHi;
                 ldsPtrHi  = tmp;
+
+                GemmDriver::scheduleKLoop();
+
+                // Make sure that all waves have finished reading / writing to lds.
+                GemmDriver::syncWorkgroup();
             }
 
             ///
@@ -272,6 +270,7 @@ namespace rocwmma
             GemmDriver::localReadA(fragsA, ldsPtrLo + ldsReadOffsetA, ldlds);
             GemmDriver::localReadB(fragsB, ldsPtrLo + ldsReadOffsetB, ldlds);
             GemmDriver::mfma(fragsAcc, fragsA, fragsB, fragsAcc);
+            GemmDriver::scheduleKLoop();
 
             ///
             /// D = alpha * accum + beta * C
