@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,6 @@
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #include <rocwmma/rocwmma.hpp>
-#include <rocwmma/rocwmma_coop.hpp>
 #include <rocwmma/rocwmma_transforms.hpp>
 #pragma GCC diagnostic pop
 
@@ -39,128 +38,9 @@ namespace rocwmma
     namespace CooperativeGemm
     {
 
-        namespace detail
-        {
-            template <typename CoopSchedulerA,
-                      typename CoopSchedulerB,
-                      uint32_t SplitCountA,
-                      uint32_t SplitCountB,
-                      bool     UseCompileTimeConstants
-                      = Schedule::WaveCountIsConstexpr<CoopSchedulerA>::value&&
-                          Schedule::WaveCountIsConstexpr<CoopSchedulerB>::value>
-            struct CoopApiSelector;
+#define GemmDriverT typename GlobalMapping, typename LdsMapping
 
-            template <typename CoopSchedulerA,
-                      typename CoopSchedulerB,
-                      uint32_t SplitCountA,
-                      uint32_t SplitCountB>
-            struct CoopApiSelector<CoopSchedulerA, CoopSchedulerB, SplitCountA, SplitCountB, true>
-            {
-                template <typename GRFragA>
-                __device__ static inline void globalReadCoopA(GRFragA&                      grFragA,
-                                                              GetDataType_t<GRFragA> const* gAddrA,
-                                                              uint32_t                      lda)
-                {
-                    rocwmma::template load_matrix_coop_sync<CoopSchedulerA::waveCount(),
-                                                            SplitCountA>(
-                        grFragA, gAddrA, lda, CoopSchedulerA::waveIndex());
-                }
-
-                template <typename GRFragB>
-                __device__ static inline void globalReadCoopB(GRFragB&                      grFragB,
-                                                              GetDataType_t<GRFragB> const* gAddrB,
-                                                              uint32_t                      ldb)
-                {
-                    rocwmma::template load_matrix_coop_sync<CoopSchedulerB::waveCount(),
-                                                            SplitCountB>(
-                        grFragB, gAddrB, ldb, CoopSchedulerB::waveIndex());
-                }
-
-                template <typename LWFragA>
-                __device__ static inline void localWriteCoopA(GetDataType_t<LWFragA>* ldsAddr,
-                                                              LWFragA const&          lwFragA,
-                                                              uint32_t                ldlds)
-                {
-                    rocwmma::template store_matrix_coop_sync<CoopSchedulerA::waveCount(),
-                                                             SplitCountA>(
-                        ldsAddr, lwFragA, ldlds, CoopSchedulerA::waveIndex());
-                }
-
-                template <typename LWFragB>
-                __device__ static inline void localWriteCoopB(GetDataType_t<LWFragB>* ldsAddr,
-                                                              LWFragB const&          lwFragB,
-                                                              uint32_t                ldlds)
-                {
-                    rocwmma::template store_matrix_coop_sync<CoopSchedulerB::waveCount(),
-                                                             SplitCountB>(
-                        ldsAddr, lwFragB, ldlds, CoopSchedulerB::waveIndex());
-                }
-            };
-
-            template <typename CoopSchedulerA,
-                      typename CoopSchedulerB,
-                      uint32_t SplitCountA,
-                      uint32_t SplitCountB>
-            struct CoopApiSelector<CoopSchedulerA, CoopSchedulerB, SplitCountA, SplitCountB, false>
-            {
-                template <typename GRFragA>
-                __device__ static inline void globalReadCoopA(GRFragA&                      grFragA,
-                                                              GetDataType_t<GRFragA> const* gAddrA,
-                                                              uint32_t                      lda)
-                {
-                    rocwmma::load_matrix_coop_sync(grFragA,
-                                                   gAddrA,
-                                                   lda,
-                                                   CoopSchedulerA::waveIndex(),
-                                                   CoopSchedulerA::waveCount(),
-                                                   SplitCountA);
-                }
-
-                template <typename GRFragB>
-                __device__ static inline void globalReadCoopB(GRFragB&                      grFragB,
-                                                              GetDataType_t<GRFragB> const* gAddrB,
-                                                              uint32_t                      ldb)
-                {
-                    rocwmma::load_matrix_coop_sync(grFragB,
-                                                   gAddrB,
-                                                   ldb,
-                                                   CoopSchedulerB::waveIndex(),
-                                                   CoopSchedulerB::waveCount(),
-                                                   SplitCountB);
-                }
-
-                template <typename LWFragA>
-                __device__ static inline void localWriteCoopA(GetDataType_t<LWFragA>* ldsAddr,
-                                                              LWFragA const&          lwFragA,
-                                                              uint32_t                ldlds)
-                {
-                    rocwmma::store_matrix_coop_sync(ldsAddr,
-                                                    lwFragA,
-                                                    ldlds,
-                                                    CoopSchedulerA::waveIndex(),
-                                                    CoopSchedulerA::waveCount(),
-                                                    SplitCountA);
-                }
-
-                template <typename LWFragB>
-                __device__ static inline void localWriteCoopB(GetDataType_t<LWFragB>* ldsAddr,
-                                                              LWFragB const&          lwFragB,
-                                                              uint32_t                ldlds)
-                {
-                    rocwmma::store_matrix_coop_sync(ldsAddr,
-                                                    lwFragB,
-                                                    ldlds,
-                                                    CoopSchedulerB::waveIndex(),
-                                                    CoopSchedulerB::waveCount(),
-                                                    SplitCountB);
-                }
-            };
-        }
-
-#define GemmDriverT \
-    typename GlobalMapping, typename LdsMapping, typename CoopSchedulerA, typename CoopSchedulerB
-
-#define GemmDriverT_impl GlobalMapping, LdsMapping, CoopSchedulerA, CoopSchedulerB
+#define GemmDriverT_impl GlobalMapping, LdsMapping
 
         template <GemmDriverT>
         template <uint32_t BlocksX>
@@ -179,9 +59,7 @@ namespace rocwmma
         __device__ inline void GemmDriver<GemmDriverT_impl>::globalReadCoopA(
             GRFragA& grFragA, GetDataType_t<GRFragA> const* gAddrA, uint32_t lda)
         {
-            using CoopApiSelector
-                = detail::CoopApiSelector<CoopSchedulerA, CoopSchedulerB, splitCountA, splitCountB>;
-            CoopApiSelector::globalReadCoopA(grFragA, gAddrA, lda);
+            rocwmma::load_matrix_sync(grFragA, gAddrA, lda);
         }
 
         template <GemmDriverT>
@@ -201,9 +79,7 @@ namespace rocwmma
         __device__ inline void GemmDriver<GemmDriverT_impl>::globalReadCoopB(
             GRFragB& grFragB, GetDataType_t<GRFragB> const* gAddrB, uint32_t ldb)
         {
-            using CoopApiSelector
-                = detail::CoopApiSelector<CoopSchedulerA, CoopSchedulerB, splitCountA, splitCountB>;
-            CoopApiSelector::globalReadCoopB(grFragB, gAddrB, ldb);
+            rocwmma::load_matrix_sync(grFragB, gAddrB, ldb);
         }
 
         template <GemmDriverT>
@@ -223,12 +99,7 @@ namespace rocwmma
         __device__ inline void GemmDriver<GemmDriverT_impl>::localWriteCoopA(
             GetDataType_t<GRFragA>* ldsAddr, GRFragA const& grFragA, uint32_t ldlds)
         {
-            using CoopApiSelector
-                = detail::CoopApiSelector<CoopSchedulerA, CoopSchedulerB, splitCountA, splitCountB>;
-            CoopApiSelector::localWriteCoopA(
-                ldsAddr,
-                LdsMapping::template formatLWFragA<CoopSchedulerA::waveCount()>(grFragA),
-                ldlds);
+            rocwmma::store_matrix_sync(ldsAddr, LdsMapping::formatLWFragA(grFragA), ldlds);
         }
 
         template <GemmDriverT>
@@ -248,19 +119,16 @@ namespace rocwmma
         __device__ inline void GemmDriver<GemmDriverT_impl>::localWriteCoopB(
             GetDataType_t<GRFragB>* ldsAddr, GRFragB const& grFragB, uint32_t ldlds)
         {
-            using CoopApiSelector
-                = detail::CoopApiSelector<CoopSchedulerA, CoopSchedulerB, splitCountA, splitCountB>;
-            CoopApiSelector::localWriteCoopB(
-                ldsAddr,
-                LdsMapping::template formatLWFragB<CoopSchedulerB::waveCount()>(grFragB),
-                ldlds);
+            rocwmma::store_matrix_sync(ldsAddr, LdsMapping::formatLWFragB(grFragB), ldlds);
         }
 
         template <GemmDriverT>
         __device__ inline void GemmDriver<GemmDriverT_impl>::localReadA(
             MfmaFragA& fragsA, GetDataType_t<MfmaFragA> const* ldsAddrA, uint32_t ldlds)
         {
-            rocwmma::load_matrix_sync(reinterpret_cast<LRFragA&>(fragsA), ldsAddrA, ldlds);
+            LRFragA lrFragA;
+            rocwmma::load_matrix_sync(lrFragA, ldsAddrA, ldlds);
+            fragsA.mStorage = LdsMapping::formatLRFragA(lrFragA).mStorage;
         }
 
         template <GemmDriverT>
@@ -281,7 +149,9 @@ namespace rocwmma
         __device__ inline void GemmDriver<GemmDriverT_impl>::localReadB(
             MfmaFragB& fragsB, GetDataType_t<MfmaFragB> const* ldsAddrB, uint32_t ldlds)
         {
-            rocwmma::load_matrix_sync(reinterpret_cast<LRFragB&>(fragsB), ldsAddrB, ldlds);
+            LRFragB lrFragB;
+            rocwmma::load_matrix_sync(lrFragB, ldsAddrB, ldlds);
+            fragsB.mStorage = LdsMapping::formatLRFragB(lrFragB).mStorage;
         }
 
         template <GemmDriverT>
@@ -424,12 +294,18 @@ namespace rocwmma
                                                      GetDataType_t<MfmaFragAcc> beta,
                                                      MfmaFragC const&           fragC)
         {
-            for(int i = 0; i < fragD.num_elements; i++)
+            // Split into chunks to not overwhelm the register capacity
+            constexpr uint32_t ChunkSize = min(8u, MfmaFragD::size());
+            for(int i = 0; i < MfmaFragD::size() / ChunkSize; i++)
             {
-                // Perform computation in ComputeT and cast back to OutputT
-                fragD.x[i] = static_cast<GetDataType_t<MfmaFragD>>(
-                    alpha * fragAcc.x[i]
-                    + beta * static_cast<GetDataType_t<MfmaFragAcc>>(fragC.x[i]));
+                for(int j = 0; j < ChunkSize; j++)
+                {
+                    auto idx = i * ChunkSize + j;
+                    // Perform computation in ComputeT and cast back to OutputT
+                    fragD.x[idx] = static_cast<GetDataType_t<MfmaFragD>>(
+                        alpha * fragAcc.x[idx]
+                        + beta * static_cast<GetDataType_t<MfmaFragAcc>>(fragC.x[idx]));
+                }
             }
         }
 
@@ -499,6 +375,20 @@ namespace rocwmma
             WaitLgkmcnt::exec();
         }
 
+        template <GemmDriverT>
+        __device__ inline void GemmDriver<GemmDriverT_impl>::scheduleKLoop()
+        {
+            // K-loop rough scheduling:
+            // 1. GR
+            // 2. LR
+            // 3. Mma
+            // 4. LW
+            SchedBarrierVmemRead::exec();
+            SchedBarrierDsRead::exec();
+            SchedBarrierMma::exec();
+            SchedBarrierDsWrite::exec();
+        }
+
 #undef GemmDriverT
 #undef GemmDriverT_impl
 
