@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,6 @@
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #include <rocwmma/rocwmma.hpp>
-#include <rocwmma/rocwmma_coop.hpp>
 #include <rocwmma/rocwmma_transforms.hpp>
 #pragma GCC diagnostic pop
 
@@ -82,17 +81,18 @@ namespace rocwmma
             // GRFragA Transposed
             // GRFragB unchanged
             // Ensure LW frags are LayoutLds friendly.
-            using LWFragA
-                = ApplyDataLayout_t<ApplyTranspose_t<typename GlobalMapping::GRFragA>, LayoutLds>;
-            using LWFragB = ApplyDataLayout_t<typename GlobalMapping::GRFragB, LayoutLds>;
+            using LWFragA = apply_data_layout_t<apply_transpose_t<typename GlobalMapping::GRFragA>,
+                                                LayoutLds>;
+            using LWFragB = apply_data_layout_t<typename GlobalMapping::GRFragB, LayoutLds>;
 
             /// LOCAL READ -> Mfma frags
             // MfmaFragA transposed
             // MfmaFragB unchanged
             // Ensure LR frags are LayoutLds friendly.
             using LRFragA
-                = ApplyDataLayout_t<ApplyTranspose_t<typename GlobalMapping::MfmaFragA>, LayoutLds>;
-            using LRFragB = ApplyDataLayout_t<typename GlobalMapping::MfmaFragB, LayoutLds>;
+                = apply_data_layout_t<apply_transpose_t<typename GlobalMapping::MfmaFragA>,
+                                      LayoutLds>;
+            using LRFragB = apply_data_layout_t<typename GlobalMapping::MfmaFragB, LayoutLds>;
 
             /// Sanity checks:
             // All local R/W tiles should have same height
@@ -140,19 +140,32 @@ namespace rocwmma
             // Leading dimension of lds matrix
             __device__ constexpr static inline auto ldLds();
 
-            template <uint32_t WaveCount = 1>
             __device__ constexpr static inline auto
                 formatLWFragA(typename GlobalMapping::GRFragA const& grFragA)
             {
-                return rocwmma::template applyDataLayout<LayoutLds, WaveCount>(
-                    applyTranspose(grFragA));
+                return apply_data_layout<LayoutLds>(apply_transpose(grFragA));
             }
 
-            template <uint32_t WaveCount = 1>
             __device__ constexpr static inline auto
                 formatLWFragB(typename GlobalMapping::GRFragB const& grFragB)
             {
-                return rocwmma::template applyDataLayout<LayoutLds, WaveCount>(grFragB);
+                return apply_data_layout<LayoutLds>(grFragB);
+            }
+
+            __device__ constexpr static inline auto formatLRFragA(LRFragA const& lrFragA)
+            {
+                using MfmaLayout =
+                    typename GetDataLayout_t<typename GlobalMapping::MfmaFragA>::Orientation;
+
+                return apply_data_layout<MfmaLayout>(apply_transpose(lrFragA));
+            }
+
+            __device__ constexpr static inline auto formatLRFragB(LRFragB const& lrFragB)
+            {
+                using MfmaLayout =
+                    typename GetDataLayout_t<typename GlobalMapping::MfmaFragB>::Orientation;
+
+                return apply_data_layout<MfmaLayout>(lrFragB);
             }
         };
 
@@ -212,17 +225,18 @@ namespace rocwmma
             // GRFragA unchanged
             // GRFragB transposed
             // Ensure LW frags are LayoutLds friendly.
-            using LWFragA = ApplyDataLayout_t<typename GlobalMapping::GRFragA, LayoutLds>;
-            using LWFragB
-                = ApplyDataLayout_t<ApplyTranspose_t<typename GlobalMapping::GRFragB>, LayoutLds>;
+            using LWFragA = apply_data_layout_t<typename GlobalMapping::GRFragA, LayoutLds>;
+            using LWFragB = apply_data_layout_t<apply_transpose_t<typename GlobalMapping::GRFragB>,
+                                                LayoutLds>;
 
             /// LOCAL READ -> Mfma frags
             // MfmaFragA unchanged
             // MfmaFragB transposed
             // Ensure LR frags are LayoutLds friendly.
-            using LRFragA = ApplyDataLayout_t<typename GlobalMapping::MfmaFragA, LayoutLds>;
+            using LRFragA = apply_data_layout_t<typename GlobalMapping::MfmaFragA, LayoutLds>;
             using LRFragB
-                = ApplyDataLayout_t<ApplyTranspose_t<typename GlobalMapping::MfmaFragB>, LayoutLds>;
+                = apply_data_layout_t<apply_transpose_t<typename GlobalMapping::MfmaFragB>,
+                                      LayoutLds>;
 
             // Sanity check:
             // All local R/W tiles should have same width
@@ -270,18 +284,32 @@ namespace rocwmma
             // Leading dimension of shared memory usage
             __device__ constexpr static inline auto ldLds();
 
-            template <uint32_t WaveCount = 1>
             __device__ constexpr static inline auto
                 formatLWFragA(typename GlobalMapping::GRFragA const& grFragA)
             {
-                return applyDataLayout<LayoutLds, WaveCount>(grFragA);
+                return apply_data_layout<LayoutLds>(grFragA);
             }
 
-            template <uint32_t WaveCount = 1>
             __device__ constexpr static inline auto
                 formatLWFragB(typename GlobalMapping::GRFragB const& grFragB)
             {
-                return applyDataLayout<LayoutLds, WaveCount>(applyTranspose(grFragB));
+                return apply_data_layout<LayoutLds>(apply_transpose(grFragB));
+            }
+
+            __device__ constexpr static inline auto formatLRFragA(LRFragA const& lrFragA)
+            {
+                using MfmaLayout =
+                    typename GetDataLayout_t<typename GlobalMapping::MfmaFragA>::Orientation;
+
+                return apply_data_layout<MfmaLayout>(lrFragA);
+            }
+
+            __device__ constexpr static inline auto formatLRFragB(LRFragB const& lrFragB)
+            {
+                using MfmaLayout =
+                    typename GetDataLayout_t<typename GlobalMapping::MfmaFragB>::Orientation;
+
+                return apply_data_layout<MfmaLayout>(apply_transpose(lrFragB));
             }
         };
 
@@ -352,15 +380,21 @@ namespace rocwmma
             // GRFragA transformed to register file
             // GRFragB transformed to register file
             // Ensure LW frags are LayoutLds friendly.
-            using LWFragA = ApplyDataLayout_t<ApplyRegisterFile_t<typename GlobalMapping::GRFragA>,
-                                              LayoutLds>;
-            using LWFragB = ApplyDataLayout_t<ApplyRegisterFile_t<typename GlobalMapping::GRFragB>,
-                                              LayoutLds>;
+            using LWFragA
+                = apply_data_layout_t<apply_register_file_t<typename GlobalMapping::GRFragA>,
+                                      LayoutLds>;
+            using LWFragB
+                = apply_data_layout_t<apply_register_file_t<typename GlobalMapping::GRFragB>,
+                                      LayoutLds>;
 
             /// LOCAL READ -> Mfma frags
             // Same as LWFrags
-            using LRFragA = LWFragA;
-            using LRFragB = LWFragB;
+            using LRFragA
+                = apply_data_layout_t<apply_register_file_t<typename GlobalMapping::MfmaFragA>,
+                                      LayoutLds>;
+            using LRFragB
+                = apply_data_layout_t<apply_register_file_t<typename GlobalMapping::MfmaFragB>,
+                                      LayoutLds>;
 
             /// Sanity checks:
             // All local R/W tiles should have same width
@@ -373,12 +407,18 @@ namespace rocwmma
 
             // This layout is only valid if global reads are MFMA friendly, due to register file transform.
             // In-register layout for MFMA blocks is not the same as wave tile or macro tile.
-            static_assert(std::is_same<typename GlobalMapping::GRFragA,
-                                       typename GlobalMapping::MfmaFragA>::value,
-                          "GR A block must be MFMA size");
-            static_assert(std::is_same<typename GlobalMapping::GRFragB,
-                                       typename GlobalMapping::MfmaFragB>::value,
-                          "GR B block must be MFMA size");
+            static_assert(
+                GetIOShape_t<typename GlobalMapping::GRFragA>::BlockWidth
+                        == GetIOShape_t<typename GlobalMapping::MfmaFragA>::BlockWidth
+                    && GetIOShape_t<typename GlobalMapping::GRFragA>::BlockHeight
+                           == GetIOShape_t<typename GlobalMapping::MfmaFragA>::BlockHeight,
+                "GR A block must be MFMA size");
+            static_assert(
+                GetIOShape_t<typename GlobalMapping::GRFragB>::BlockWidth
+                        == GetIOShape_t<typename GlobalMapping::MfmaFragB>::BlockWidth
+                    && GetIOShape_t<typename GlobalMapping::GRFragB>::BlockHeight
+                           == GetIOShape_t<typename GlobalMapping::MfmaFragB>::BlockHeight,
+                "GR B block must be MFMA size");
 
         private: // Coordinate projection helpers
             constexpr static uint32_t LdsWidth = Constants::AMDGCN_WAVE_SIZE;
@@ -416,8 +456,8 @@ namespace rocwmma
                 // When interpreting as a register block (e.g. BlockDim 64 on CDNA), avoid transforming
                 // direct register layouts because we want the register contents as they currently are.
                 return reinterpret_cast<
-                    ApplyDataLayout_t<ApplyRegisterFile_t<typename GlobalMapping::GRFragA>,
-                                      LayoutLds> const&>(grFragA);
+                    apply_data_layout_t<apply_register_file_t<typename GlobalMapping::GRFragA>,
+                                        LayoutLds> const&>(grFragA);
             }
 
             template <uint32_t WaveCount = 1>
@@ -427,8 +467,24 @@ namespace rocwmma
                 // When interpreting as a register block (e.g. BlockDim 64 on CDNA), avoid transforming
                 // direct register layouts because we want the register contents as they currently are.
                 return reinterpret_cast<
-                    ApplyDataLayout_t<ApplyRegisterFile_t<typename GlobalMapping::GRFragB>,
-                                      LayoutLds> const&>(grFragB);
+                    apply_data_layout_t<apply_register_file_t<typename GlobalMapping::GRFragB>,
+                                        LayoutLds> const&>(grFragB);
+            }
+
+            __device__ constexpr static inline auto formatLRFragA(LRFragA const& lrFragA)
+            {
+                using MfmaLayout =
+                    typename GetDataLayout_t<typename GlobalMapping::MfmaFragA>::Orientation;
+
+                return apply_data_layout<MfmaLayout>(lrFragA);
+            }
+
+            __device__ constexpr static inline auto formatLRFragB(LRFragB const& lrFragB)
+            {
+                using MfmaLayout =
+                    typename GetDataLayout_t<typename GlobalMapping::MfmaFragB>::Orientation;
+
+                return apply_data_layout<MfmaLayout>(lrFragB);
             }
         };
 
