@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -121,11 +121,15 @@ namespace rocwmma
               || (bool)TestTraits::InputType::IsFloat64,
 
             // Gfx940/1/2 arch req'd for float8_fnuz_t, bfloat8_fnuz_t and xfloat32_t
-            F8XF32ArchTest = !((bool)TestTraits::InputType::IsFloat8Fnuz
-                               || (bool)TestTraits::InputType::IsBFloat8Fnuz
-                               || (bool)TestTraits::InputType::IsXFloat32)
-                             || (bool)TestTraits::Arch::IsGfx940 || (bool)TestTraits::Arch::IsGfx941
-                             || (bool)TestTraits::Arch::IsGfx942,
+            F8FnuzXF32ArchTest = !((bool)TestTraits::InputType::IsFloat8Fnuz
+                                   || (bool)TestTraits::InputType::IsBFloat8Fnuz
+                                   || (bool)TestTraits::InputType::IsXFloat32)
+                                 || (bool)TestTraits::Arch::IsGfx942,
+
+            // Gfx950 arch req'd for float8_t
+            F8ArchTest
+            = !((bool)TestTraits::InputType::IsFloat8 || (bool)TestTraits::InputType::IsBFloat8)
+              || (bool)TestTraits::Arch::IsGfx950,
 
             // All archs except gfx908 can run float64_t
             F64ArchTest
@@ -137,16 +141,15 @@ namespace rocwmma
             I8BlockSizeTest = !((bool)TestTraits::InputType::IsInt8)
                               || ((bool)TestTraits::BlockSizes::isBlockMN16 && (BlockK >= 16u)
                                   && (BlockK % 16u == 0u))
-                              || ((bool)TestTraits::BlockSizes::isBlockMN32 && (BlockK >= 8u)
-                                  && (BlockK % 8u == 0u)),
+                              || ((bool)TestTraits::BlockSizes::isBlockMN32 && (BlockK >= 16u)
+                                  && (BlockK % 16u == 0u)),
 
-            // Follow-on to gfx940/1/2 int8_t.
+            // Follow-on to gfx940/1/2/950 int8_t.
             // BlockM/N = 16; Block K >= 32
             // BlockM/N = 32; Block K >= 16
-            Gfx940I8BlockSizeTest
+            ExtI8BlockSizeTest
             = !((bool)TestTraits::InputType::IsInt8
-                && ((bool)TestTraits::Arch::IsGfx940 || (bool)TestTraits::Arch::IsGfx941
-                    || (bool)TestTraits::Arch::IsGfx942))
+                && ((bool)TestTraits::Arch::IsGfx942 || (bool)TestTraits::Arch::IsGfx950))
               || ((bool)TestTraits::BlockSizes::isBlockMN16 && (BlockK >= 32u)
                   && (BlockK % 32u == 0u))
               || ((bool)TestTraits::BlockSizes::isBlockMN32 && (BlockK >= 16u)
@@ -171,8 +174,8 @@ namespace rocwmma
             = !((bool)TestTraits::InputType::IsFloat16 || (bool)TestTraits::InputType::IsBFloat16)
               || ((bool)TestTraits::BlockSizes::isBlockMN16 && (BlockK >= 16u)
                   && (BlockK % 16u == 0u))
-              || ((bool)TestTraits::BlockSizes::isBlockMN32 && (BlockK >= 8u)
-                  && (BlockK % 8u == 0u)),
+              || ((bool)TestTraits::BlockSizes::isBlockMN32 && (BlockK >= 16u)
+                  && (BlockK % 16u == 0u)),
 
             // Older gfx908 arch has half BlockK on bfloat16_t
             // BlockM/N = 16; Block K >= 8
@@ -190,8 +193,8 @@ namespace rocwmma
             F32BlockSizeTest = !((bool)TestTraits::InputType::IsFloat32)
                                || ((bool)TestTraits::BlockSizes::isBlockMN16 && (BlockK >= 4u)
                                    && (BlockK % 4u == 0u))
-                               || ((bool)TestTraits::BlockSizes::isBlockMN32 && (BlockK >= 2u)
-                                   && (BlockK % 2u == 0u)),
+                               || ((bool)TestTraits::BlockSizes::isBlockMN32 && (BlockK >= 8u)
+                                   && (BlockK % 8u == 0u)),
 
             // General xfloat32_t block size
             // BlockM/N = 16; Block K >= 8
@@ -208,10 +211,10 @@ namespace rocwmma
                                || ((bool)TestTraits::BlockSizes::isBlockMN16 && (BlockK >= 4u)
                                    && (BlockK % 4u == 0u)),
 
-            Enable = (ArchTest && WaveSizeTest && TBlockTest && InputTypesTest && F8XF32ArchTest
-                      && F64ArchTest && I8BlockSizeTest && Gfx940I8BlockSizeTest && F8BlockSizeTest
-                      && F16BlockSizeTest && Gfx908BF16BlockSizeTest && F32BlockSizeTest
-                      && XF32BlockSizeTest && F64BlockSizeTest)
+            Enable = (ArchTest && WaveSizeTest && TBlockTest && InputTypesTest && F8FnuzXF32ArchTest
+                      && F8ArchTest && F64ArchTest && I8BlockSizeTest && ExtI8BlockSizeTest
+                      && F8BlockSizeTest && F16BlockSizeTest && Gfx908BF16BlockSizeTest
+                      && F32BlockSizeTest && XF32BlockSizeTest && F64BlockSizeTest)
         };
 
 #if !NDEBUG
