@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -30,37 +30,25 @@
 #include <rocwmma/rocwmma.hpp>
 
 #include "common.hpp"
+#include "unit_test_macros.hpp"
+
+static constexpr bool DebugOnFail = false;
+
+#define PACK_UTIL_EXPECT_EQ(LHS, RHS) ROCWMMA_EXPECT_EQUAL(LHS, RHS, DebugOnFail);
 
 namespace rocwmma
 {
-    template <typename DataT, uint32_t VecSize>
-    ROCWMMA_DEVICE static inline DataT get(VecT<DataT, VecSize> const& v, uint32_t idx)
-    {
-        return v.data[idx];
-    }
-
-    template <typename DataT, uint32_t VecSize>
-    ROCWMMA_DEVICE static inline auto generateSeqVec()
-    {
-        auto buildSeq = [](auto&& idx) {
-            constexpr auto Index = std::decay_t<decltype(idx)>::value;
-            return static_cast<DataT>(Index);
-        };
-
-        return vector_generator<DataT, VecSize>()(buildSeq);
-    }
-
     template <typename DataT, uint32_t VecSize>
     ROCWMMA_DEVICE static inline bool packUtilTestBasic()
     {
         bool err = false;
 
-        auto res = generateSeqVec<DataT, VecSize>();
+        auto res = make_vector_sequence<DataT, VecSize>();
 
-        for(uint32_t i = 0; i < VecSize; i++)
-        {
-            err |= (get(res, i) != static_cast<DataT>(i));
-        }
+        static_for<0, VecSize, 1>([&](auto&& Idx) {
+            constexpr uint32_t i = decay_t<decltype(Idx)>::value;
+            err |= !PACK_UTIL_EXPECT_EQ((get<i>(res)), static_cast<DataT>(i));
+        });
 
         return err;
     }
@@ -77,11 +65,11 @@ namespace rocwmma
         using PackUtil = rocwmma::PackUtil<DataT>;
 
         bool err = false;
-        auto res = generateSeqVec<DataT, VecSize>();
+        auto res = make_vector_sequence<DataT, VecSize>();
 
         if constexpr(PackUtil::Traits::PackRatio == 1)
         {
-            err |= res != PackUtil::pad(res);
+            err |= !PACK_UTIL_EXPECT_EQ(res, PackUtil::pad(res));
         }
         else
         {
@@ -89,14 +77,18 @@ namespace rocwmma
             using UnpackedT = typename PackUtil::Traits::UnpackedT;
 
             auto paddedData = PackUtil::pad(res);
-            err |= !std::is_same_v<decltype(paddedData), VecT<PackedT, VecSize>>;
-            for(uint32_t i = 0; i < VecSize; i++)
-            {
-                auto value = get(paddedData, i);
+            err |= !PACK_UTIL_EXPECT_EQ(
+                (std::is_same_v<decltype(paddedData), VecT<PackedT, VecSize>>), true);
+
+            static_for<0, VecSize, 1>([&](auto&& Idx) {
+                constexpr uint32_t i = decay_t<decltype(Idx)>::value;
+
+                auto value = get<i>(paddedData);
 
                 // unpadded data is at idx 0 in the padded data by default
-                err |= *reinterpret_cast<DataT*>(&value) != static_cast<DataT>((float)i);
-            }
+                err |= !PACK_UTIL_EXPECT_EQ(*reinterpret_cast<DataT*>(&value),
+                                            static_cast<DataT>((float)i));
+            });
         }
 
         return err;
@@ -114,11 +106,11 @@ namespace rocwmma
         using PackUtil = rocwmma::PackUtil<DataT>;
 
         bool err = false;
-        auto res = generateSeqVec<DataT, VecSize>();
+        auto res = make_vector_sequence<DataT, VecSize>();
 
         if constexpr(PackUtil::Traits::PackRatio == 1)
         {
-            err |= res != PackUtil::pad(res);
+            err |= !PACK_UTIL_EXPECT_EQ(res, PackUtil::pad(res));
         }
         else
         {
@@ -127,12 +119,16 @@ namespace rocwmma
 
             // padIdx == 1 which is valid for 8 bits and 16 bits
             auto paddedData = PackUtil::template pad<1>(res);
-            err |= !std::is_same_v<decltype(paddedData), VecT<PackedT, VecSize>>;
-            for(uint32_t i = 0; i < VecSize; i++)
-            {
-                auto value = get(paddedData, i);
-                err |= *(reinterpret_cast<DataT*>(&value) + 1) != static_cast<DataT>((float)i);
-            }
+            err |= !PACK_UTIL_EXPECT_EQ(
+                (std::is_same_v<decltype(paddedData), VecT<PackedT, VecSize>>), true);
+
+            static_for<0, VecSize, 1>([&](auto&& Idx) {
+                constexpr uint32_t i = decay_t<decltype(Idx)>::value;
+
+                PackedT value = get<i>(paddedData);
+                err |= !PACK_UTIL_EXPECT_EQ(*(reinterpret_cast<UnpackedT*>(&value) + 1u),
+                                            static_cast<UnpackedT>(i));
+            });
         }
 
         return err;
@@ -155,21 +151,22 @@ namespace rocwmma
 
         if constexpr(PackUtil::Traits::PackRatio == 1)
         {
-            auto res = generateSeqVec<PackedT, VecSize>();
-            err |= res != PackUtil::unpad(res);
+            auto res = make_vector_sequence<PackedT, VecSize>();
+            err |= !PACK_UTIL_EXPECT_EQ(res, PackUtil::unpad(res));
         }
         else
         {
-            auto res = VecT<PackedT, VecSize>(0);
-            for(uint32_t i = 0; i < VecSize; i++)
-            {
-                *reinterpret_cast<UnpackedT*>(reinterpret_cast<PackedT*>(&res.data) + i)
-                    = static_cast<UnpackedT>((float)i);
-            }
+            auto res = make_vector<PackedT, VecSize>(0);
+
+            static_for<0, VecSize, 1>([&](auto&& Idx) {
+                constexpr uint32_t i                          = decay_t<decltype(Idx)>::value;
+                *reinterpret_cast<UnpackedT*>(&(get<i>(res))) = static_cast<UnpackedT>((float)i);
+            });
 
             auto unpaddedData = PackUtil::unpad(res);
-            err |= !std::is_same_v<decltype(unpaddedData), VecT<UnpackedT, VecSize>>;
-            err |= unpaddedData != generateSeqVec<DataT, VecSize>();
+            err |= !PACK_UTIL_EXPECT_EQ(
+                (std::is_same_v<decltype(unpaddedData), VecT<UnpackedT, VecSize>>), true);
+            err |= !PACK_UTIL_EXPECT_EQ(unpaddedData, (make_vector_sequence<DataT, VecSize>()));
         }
 
         return err;
@@ -192,21 +189,23 @@ namespace rocwmma
 
         if constexpr(PackUtil::Traits::PackRatio == 1)
         {
-            auto res = generateSeqVec<PackedT, VecSize>();
-            err |= res != PackUtil::unpad(res);
+            auto res = make_vector_sequence<PackedT, VecSize>();
+            err |= !PACK_UTIL_EXPECT_EQ(res, PackUtil::unpad(res));
         }
         else
         {
-            auto res = VecT<PackedT, VecSize>(0);
-            for(uint32_t i = 0; i < VecSize; i++)
-            {
-                *(reinterpret_cast<UnpackedT*>(reinterpret_cast<PackedT*>(&res.data) + i) + 1)
+            auto res = make_vector<PackedT, VecSize>(0);
+
+            static_for<0, VecSize, 1>([&](auto&& Idx) {
+                constexpr uint32_t i = decay_t<decltype(Idx)>::value;
+                *(reinterpret_cast<UnpackedT*>(&(get<i>(res))) + 1)
                     = static_cast<UnpackedT>((float)i);
-            }
+            });
 
             auto unpaddedData = PackUtil::template unpad<1>(res);
-            err |= !std::is_same_v<decltype(unpaddedData), VecT<UnpackedT, VecSize>>;
-            err |= unpaddedData != generateSeqVec<DataT, VecSize>();
+            err |= !PACK_UTIL_EXPECT_EQ(
+                (std::is_same_v<decltype(unpaddedData), VecT<UnpackedT, VecSize>>), true);
+            err |= !PACK_UTIL_EXPECT_EQ(unpaddedData, (make_vector_sequence<DataT, VecSize>()));
         }
 
         return err;
@@ -224,15 +223,17 @@ namespace rocwmma
         // required by pack(): VecSize % PackUtil::Traits::PackRatio == 0
         if constexpr(VecSize % PackUtil::Traits::PackRatio == 0)
         {
-            auto res = generateSeqVec<UnpackedT, VecSize>();
+            auto res = make_vector_sequence<UnpackedT, VecSize>();
 
             // pack() cast VecT<UnpackedT, VecSize> const &  to VecT<PackedT, VecSize / PackRatio> const &
-            err |= !std::is_same_v<decltype(PackUtil::pack(res)),
-                                   VecT<PackedT, VecSize / PackUtil::Traits::PackRatio> const&>;
+            err |= !PACK_UTIL_EXPECT_EQ(
+                (std::is_same_v<decltype(PackUtil::pack(res)),
+                                VecT<PackedT, VecSize / PackUtil::Traits::PackRatio> const&>),
+                true);
 
             // argument and return of pack() point to the same addr
-            err |= reinterpret_cast<void const*>(&PackUtil::pack(res))
-                   != reinterpret_cast<void const*>(&res);
+            err |= !PACK_UTIL_EXPECT_EQ(reinterpret_cast<void const*>(&PackUtil::pack(res)),
+                                        reinterpret_cast<void const*>(&res));
         }
 
         return err;
@@ -247,15 +248,17 @@ namespace rocwmma
 
         bool err = false;
 
-        auto res = generateSeqVec<PackedT, VecSize>();
+        auto res = make_vector_sequence<PackedT, VecSize>();
 
         // pack() cast VecT<PackedT, VecSize> const &  to VecT<UnpackedT, VecSize * PackRatio> const &
-        err |= !std::is_same_v<decltype(PackUtil::unpack(res)),
-                               VecT<UnpackedT, VecSize * PackUtil::Traits::PackRatio> const&>;
+        err |= !PACK_UTIL_EXPECT_EQ(
+            (std::is_same_v<decltype(PackUtil::unpack(res)),
+                            VecT<UnpackedT, VecSize * PackUtil::Traits::PackRatio> const&>),
+            true);
 
         // argument and return of pack() point to the same addr
-        err |= reinterpret_cast<void const*>(&PackUtil::unpack(res))
-               != reinterpret_cast<void const*>(&res);
+        err |= !PACK_UTIL_EXPECT_EQ(reinterpret_cast<void const*>(&PackUtil::unpack(res)),
+                                    reinterpret_cast<void const*>(&res));
 
         return err;
     }
@@ -268,14 +271,16 @@ namespace rocwmma
         using UnpackedT = typename PackUtil::Traits::UnpackedT;
 
         bool err = false;
-        auto res = generateSeqVec<UnpackedT, VecSize>();
+        auto res = make_vector_sequence<UnpackedT, VecSize>();
 
         if constexpr(VecSize % PackUtil::Traits::PackRatio == 0)
         {
-            err |= !std::is_same_v<decltype(PackUtil::paddedPack(res)),
-                                   VecT<PackedT, VecSize / PackUtil::Traits::PackRatio> const&>;
-            err |= reinterpret_cast<void const*>(&PackUtil::paddedPack(res))
-                   != reinterpret_cast<void const*>(&res);
+            err |= !PACK_UTIL_EXPECT_EQ(
+                (std::is_same_v<decltype(PackUtil::paddedPack(res)),
+                                VecT<PackedT, VecSize / PackUtil::Traits::PackRatio> const&>),
+                true);
+            err |= !PACK_UTIL_EXPECT_EQ(reinterpret_cast<void const*>(&PackUtil::paddedPack(res)),
+                                        reinterpret_cast<void const*>(&res));
         }
         else if constexpr(VecSize * 2 == PackUtil::Traits::PackRatio)
         {
@@ -285,25 +290,28 @@ namespace rocwmma
                 *(reinterpret_cast<VecT<UnpackedT, VecSize>*>(&expectedResult) + i) = res;
             }
 
-            err |= expectedResult != PackUtil::paddedPack(res);
+            err |= !PACK_UTIL_EXPECT_EQ(expectedResult, PackUtil::paddedPack(res));
         }
         else if constexpr(VecSize == 1)
         {
             if constexpr(PackUtil::Traits::PackRatio == 1)
             {
-                err |= res != PackUtil::paddedPack(res);
+                err |= !PACK_UTIL_EXPECT_EQ(res, PackUtil::paddedPack(res));
             }
             else
             {
                 auto paddedData = PackUtil::paddedPack(res);
                 err |= !std::is_same_v<decltype(paddedData), VecT<PackedT, VecSize>>;
-                for(uint32_t i = 0; i < VecSize; i++)
-                {
-                    auto value = get(paddedData, i);
+
+                static_for<0, VecSize, 1>([&](auto&& Idx) {
+                    constexpr uint32_t i = decay_t<decltype(Idx)>::value;
+
+                    auto value = get<i>(paddedData);
 
                     // unpadded data is at idx 0 in the padded data by default
-                    err |= *reinterpret_cast<DataT*>(&value) != static_cast<DataT>((float)i);
-                }
+                    err |= !PACK_UTIL_EXPECT_EQ(*reinterpret_cast<DataT*>(&value),
+                                                static_cast<DataT>((float)i));
+                });
             }
         }
 
@@ -321,11 +329,14 @@ namespace rocwmma
 
         if constexpr(VecSize % PackUtil::Traits::PackRatio == 0)
         {
-            auto res = generateSeqVec<PackedT, VecSize / PackUtil::Traits::PackRatio>();
-            err |= !std::is_same_v<decltype(PackUtil::template paddedUnpack<VecSize>(res)),
-                                   VecT<UnpackedT, VecSize> const&>;
-            err |= reinterpret_cast<void const*>(&PackUtil::template paddedUnpack<VecSize>(res))
-                   != reinterpret_cast<void const*>(&res);
+            auto res = make_vector_sequence<PackedT, VecSize / PackUtil::Traits::PackRatio>();
+            err |= !PACK_UTIL_EXPECT_EQ(
+                (std::is_same_v<decltype(PackUtil::template paddedUnpack<VecSize>(res)),
+                                VecT<UnpackedT, VecSize> const&>),
+                true);
+            err |= !PACK_UTIL_EXPECT_EQ(
+                reinterpret_cast<void const*>(&PackUtil::template paddedUnpack<VecSize>(res)),
+                reinterpret_cast<void const*>(&res));
         }
         return err;
     }
@@ -342,13 +353,14 @@ namespace rocwmma
         if constexpr(VecSize * 2 == PackUtil::Traits::PackRatio)
         {
             auto constexpr packSize = std::max(1u, VecSize / PackUtil::Traits::PackRatio);
-            auto res                = VecT<PackedT, packSize>(0);
-            auto expectedData       = generateSeqVec<UnpackedT, VecSize>();
+            auto res                = make_vector<PackedT, packSize>(0);
+            auto expectedData       = make_vector_sequence<UnpackedT, VecSize>();
             for(uint32_t i = 0; i < VecSize; i++)
             {
-                *(reinterpret_cast<VecT<UnpackedT, VecSize>*>(&res.data) + i) = expectedData;
+                *(reinterpret_cast<VecT<UnpackedT, VecSize>*>(&res) + i) = expectedData;
             }
-            err |= PackUtil::template paddedUnpack<VecSize>(res) != expectedData;
+            err |= !PACK_UTIL_EXPECT_EQ(PackUtil::template paddedUnpack<VecSize>(res),
+                                        expectedData);
         }
         return err;
     }
@@ -364,11 +376,11 @@ namespace rocwmma
 
         if constexpr(VecSize == 1)
         {
-            auto res          = VecT<PackedT, 1>(0);
-            auto expectedData = VecT<UnpackedT, 1>(static_cast<UnpackedT>(1.0F));
-            (*(reinterpret_cast<VecT<UnpackedT, VecSize>*>(&res.data))).data[0]
-                = expectedData.data[0];
-            err |= PackUtil::template paddedUnpack<VecSize>(res) != expectedData;
+            auto res          = make_vector<PackedT, 1>(0);
+            auto expectedData = make_vector<UnpackedT, 1>(1.0f);
+            get<0>(*(reinterpret_cast<VecT<UnpackedT, VecSize>*>(&res))) = get<0>(expectedData);
+            err |= !PACK_UTIL_EXPECT_EQ(PackUtil::template paddedUnpack<VecSize>(res),
+                                        expectedData);
         }
         return err;
     }
